
package com.xdf.creative.util.file;

import com.xdf.creative.enums.ServiceFileUploadTypeEnum;
import com.xdf.creative.enums.StatusCode;
import com.xdf.creative.util.UUIDUtil;
import com.xdf.creative.util.page.ApiResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.springframework.web.multipart.MultipartFile;
import sun.misc.BASE64Decoder;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * 文件上传工具类
 *
 * @author DeanYe
 * @date 2019/8/21
 * @since 1.2.1-RELEASE
 */
@Slf4j
public final class UploadUtil {

    public static ArrayList<String> fileTypeList = new ArrayList<String>() {{
        add("PDF");
        add("DOC");
        add("DOCX");
        add("JPEG");
        add("XLS");
        add("XLSX");
        add("PPT");
        add("PPTX");
        add("PNG");
        add("JPG");
        add("TXT");
        add("GIF");
    }};

    /**
     * 上传文件，默认文件名格式，yyyyMMddHHmmssS
     *
     * @param uploadPath
     * @param multipartFile
     * @param fileType      文件类型 1文件 2图片
     * @return
     * @throws Exception
     */
    public static ApiResult upload(String uploadPath, MultipartFile multipartFile, Integer fileType) throws Exception {
        return upload(uploadPath, multipartFile, fileType, new DefaultUploadFileNameHandleImpl());
    }

    /**
     * 上传文件
     *
     * @param uploadPath           上传目录
     * @param multipartFile        上传文件
     * @param uploadFileNameHandle 回调
     * @return
     * @throws Exception
     */
    public static ApiResult upload(String uploadPath, MultipartFile multipartFile, Integer fileType, UploadFileNameHandle uploadFileNameHandle) {
        // 获取输入流
        String saveFileName = "";
        String fileExtension = FilenameUtils.getExtension(multipartFile.getOriginalFilename());
        if (!fileTypeList.contains(fileExtension.toUpperCase())) {
            return ApiResult.error(StatusCode.WEB_FILE_UPLOAD_FAILED);
        }
        try {
            InputStream inputStream = multipartFile.getInputStream();
            // 文件保存目录
            File saveDir = new File(uploadPath);
            // 判断目录是否存在，不存在，则创建，如创建失败，则抛出异常
            if (!saveDir.exists()) {
                boolean flag = saveDir.mkdirs();
                if (!flag) {
                    log.error("创建" + saveDir + "目录失败！");

                }
            }
            String originalFilename = multipartFile.getOriginalFilename();
            if (fileType.equals(ServiceFileUploadTypeEnum.SERVICE_FILE_UPLOAD_TYPE_FILE.getCode())) {
                if (uploadFileNameHandle == null) {
                    saveFileName = new DefaultUploadFileNameHandleImpl().handle(originalFilename);
                } else {
                    saveFileName = uploadFileNameHandle.handle(originalFilename);
                }
            } else {
                saveFileName = originalFilename;
            }
            log.info("saveFileName = " + saveFileName);
            File saveFile = new File(saveDir, saveFileName);
            // 保存文件到服务器指定路径
            FileUtils.copyToFile(inputStream, saveFile);
        } catch (IOException e) {
            e.printStackTrace();
            return ApiResult.error(StatusCode.WEB_FILE_DIR_PATH_CREATE_ERROR);
        }
        return ApiResult.ok(saveFileName);
    }

    /**
     * 删除删除的文件
     *
     * @param uploadPath
     * @param saveFileName
     */
    public static void deleteQuietly(String uploadPath, String saveFileName) {
        File saveDir = new File(uploadPath);
        File saveFile = new File(saveDir, saveFileName);
        log.debug("删除文件：" + saveFile);
        FileUtils.deleteQuietly(saveFile);

    }
    public static interface UploadFileNameHandle {
        /**
         * 回调处理接口
         *
         * @param originalFilename
         * @return
         */
        String handle(String originalFilename);
    }
    public static class DefaultUploadFileNameHandleImpl implements UploadFileNameHandle {
        @Override
        public String handle(String originalFilename) {
            // 文件后缀
            String fileExtension = FilenameUtils.getExtension(originalFilename);
            // 这里可自定义文件名称，比如按照业务类型/文件格式/日期
            // 此处按照文件日期存储
            String dateString = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmssS"));
            String fileName = dateString + UUIDUtil.getUUID() + "." + fileExtension;
            return fileName;
        }
    }
    /**
     * base64上传
     * @param baseCodeFile 文件编码
     * @param filePath 文件上传地址不含文件名
     * @param fileName 文件名称
     * @param returnName 返回地址含文件名
     * @param fileType 文件类型
     */
    public  static ApiResult BaseDecoderUpload(String baseCodeFile,String filePath,String fileName,String returnName,String fileType) throws Exception {
        //对内容进行base64解码
        BASE64Decoder decoder = new BASE64Decoder();
        // 生成目录
        File dirFile = new File(filePath);
        if (!dirFile.isDirectory()) {
            dirFile.mkdirs();
        }
        // 生成新的文件名
        returnName = returnName+ "/" +fileName+"."+fileType;
        String realPath = filePath + "/" +fileName+"."+fileType;
        try {
            FileOutputStream write = new FileOutputStream(new File(realPath));
            byte[] decodeBytes = decoder.decodeBuffer(baseCodeFile);
            write.write(decodeBytes);
            write.close();
            Map<String,Object> pathMap =new HashMap<String,Object>();
            pathMap.put("path",returnName);
            return ApiResult.ok(pathMap);
        } catch (IOException e) {
            return ApiResult.error(StatusCode.FILE_CL_UPLOAD_FAILED);
        }
    }
}
